@@ -29,7 +29,7 @@ use urpm::select;
 use urpm::orphans;
 use urpm::get_pkgs;
 use urpm::signature;
-use urpm::util qw(find intersection member partition);
+use urpm::util qw(any difference2 find intersection member partition untaint);
 
 #- global boolean options
 my ($auto_select, $no_install, $install_src, $clean, $noclean, $force, $parallel, $test);
@@ -217,11 +217,10 @@ sub _init_common_options {
       justdb => $options{justdb},
       replacepkgs => $options{replacepkgs},
       callback_close_helper => $callbacks->{close_helper},
-      callback_error => $callbacks->{error},
       callback_inst => $callbacks->{inst},
       callback_open_helper => $callbacks->{open_helper},
       callback_trans => $callbacks->{trans},
-      callback_uninst => $callbacks->{uninst},
+      callback_report_uninst => $callbacks->{callback_report_uninst},
       raw_message => 1,
   );
 }
@@ -310,7 +309,33 @@ sub _run_transaction {
     !$fatal;
 }
 
-=item run($urpm, $state, $something_was_to_be_done, $ask_unselect, $callbacks)
+# computes the number of packages to install or to erase -
+# side-effects: urpm->{nb_install}
+my $fullname2name_re = qr/^(.*)-[^\-]*-[^\-]*\.[^\.\-]*$/;
+sub _compute_pkg_total {
+    my ($urpm, $state, $options) = @_;
+    $urpm->{nb_install} = 0;
+
+    # first account for install/erasures we will schedule (upgrades count twice as they involve erasure of the older package):
+    foreach my $set (@{$state->{transaction} || []}) {
+	my ($install, $upgrade) = partition {
+	    my $pkg = $urpm->{depslist}[$_];
+	    $pkg && !$pkg->flag_installed;
+	} @{$set->{upgrade}};
+	my $remove = $options->{'allow-force'} ? [] : $set->{remove} || [];
+	my ($rm_count, $inst_count, $up_count) = (scalar(@$remove), scalar(values @$install), scalar(keys @$upgrade));
+	$urpm->{nb_install} += $rm_count + $inst_count + 2 * $up_count;
+    }
+
+    # account for erases added by rpm (removal through obsoletes):
+    foreach my $fn (keys %{$state->{transaction_state}{rejected} || {}}) {
+	my ($n) = $fn =~ $fullname2name_re;
+	next if !keys %{$state->{transaction_state}{rejected}{$fn}{obsoleted}};
+	$urpm->{nb_install}++ if any { !/^\Q$n/ } keys %{$state->{transaction_state}{rejected}{$fn}{obsoleted}};
+    }
+}
+
+=item run($urpm, $state, $something_was_to_be_done, $ask_unselect, $_requested, $callbacks)
 
 Run the main urpm loop:
 
@@ -346,104 +371,6 @@ Run the main urpm loop:
 
 Warning: locking is left to callers...
 
-Parameters:
-
-=over
-
-=item $urpm: the urpm object
-
-=item $state: the state object (see L<URPM>)
-
-=item $something_was_to_be_done
-
-=item $ask_unselect: an array ref of packages that could not be selected
-
-=item $callbacks: a hash ref of callbacks :
-
-=over
-
-=item packages download:
-
-=over
-
-=item trans_log($mode, $file, $percent, $total, $eta, $speed): called for displaying download progress
-
-=item post_download(): called after completing download of packages
-
-=back
-
-=item interaction with user:
-
-=over
-
-=item ask_yes_or_no($_title, $msg)
-
-=item need_restart($need_restart_formatted) called when restarting urpmi is needed (priority upgrades)
-
-=item message($_title, $msg): display a message (with a title for GUIes)
-
-=back
-
-=item signature management:
-
-=over
-
-=item pre_check_sig(): signature checking startup (for rpmdrake)
-
-=item check_sig(): signature checking progress (for rpmdrake)
-
-=item bad_signature($msg, $msg2): called when a package is not/badly signed
-
-=back
-
-=item removable media management:
-
-=over
-
-=item pre_removable(): called before handling removable media (for rpmdrake)
-
-=item copy_removable($medium_name): called for asking inserting a CD/DVD
-
-=item post_extract($set, $transaction_sources, $transaction_sources_install) called after handling removable media (for rpmdrake)
-
-=back
-
-=item packages installation callbacks (passed to urpm::install::install(), see L<URPM> for parameters)
-
-=over
-
-=item open_helper(): called when opening a package, must return a fd
-
-=item close_helper(): called when package is closed
-
-=item inst() called for package opening/progress/end
-
-=item trans() called for transaction opening/progress/end
-
-=item uninst(): called for erasure progress
-
-=item error() called for cpio, script or unpacking errors
-
-=back
-
-=item finish callbacks (mainly GUI callbacks for rpmdrake/gurpmi/drakx)
-
-=over
-
-=item completed(): called when everything is completed (for cleanups)
-
-=item trans_error_summary($nok, $errors) called to warn than $nok transactions failed with $errors messages
-
-=item success_summary() called on success
-
-=item already_installed_or_not_installable($msg1, $msg2)
-
-=back
-
-=back
-
-=back
-
 =cut
 
 
@@ -498,6 +425,9 @@ sub run {
     my $migrate_back_rpmdb_db_version = 
       $urpm->{root} && urpm::select::should_we_migrate_back_rpmdb_db_version($urpm, $state);
 
+    #- compute package total:
+    _compute_pkg_total($urpm, $state, $options);
+
     #- now process each remove/install transaction
     foreach my $set (@{$state->{transaction} || []}) {
 
